#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stddef.h>
#include "nrutil.h"
#include "global.h"
#include <gtk/gtk.h>
#include <locale.h>

#include "interface.h"
#include "support.h"
#define MAX_DATA 30000
#define NR_END 1
#define FREE_ARG char*

void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}


float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
	float *v;

	v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

//Berechnung der 2.Abl an i ueber 2 Randbedingungen und N-2 Gleichungen der ersten Ableitungen an x = xi aus [xi-1,xi] und [xi,xi-1]

void spline(float x[], float y[], int n, float yp1, float ypn, float y2[])
{
	int i,k;
	float p,qn,sig,un,*u;

	u=vector(1,n-1);
	if (yp1 > 0.99e30)
		y2[1]=u[1]=0.0;
	else {
		y2[1] = -0.5;
		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
	}
	for (i=2;i<=n-1;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
	}
	y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
	for (k=n-1;k>=1;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
	free_vector(u,1,n-1);
}

//Berechnung des durch die kubische Spline interpolierten Wertes y(x)
float splint(float xa[], float ya[], float y2a[], int n, float x)
{
	void nrerror(char error_text[]);
	int klo, khi, k;
	float h, b, a, y;
	
	klo=1;
	khi=n;
	while(khi-klo>1){			//Bestimmung der Stellen ober- und unterhalb des zu interpolierenden Wertes
		k=(khi+klo) >> 1;
		if(xa[k]>x) 
			khi=k;
		else
			klo=k;
	}
	
	h=xa[khi]-xa[klo];			//Differenz zwischen den umgebenden Stuetzstellen
	if(h==0.0) 
		nrerror("Differenz zwischen den Stuetzstellen muss > 0 sein!");
	a=(xa[khi]-x)/h;			//entsprechen A und B bzw. im Weiteren C und D
	b=(x-xa[klo])/h;
	y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;	//Gl. des kub. Splines
	return y;
}

//Darstellung der Wertepaare mit Grenzen in GNUPLOT
void plotprint(char datei[], char datendatei[], char x[], char y[], double max_x, double max_y, double a, double b, double phi){
	FILE *eingabe;
	char plotbefehl[100];
	eingabe=fopen(datei,"w");
	fprintf(eingabe,"reset\n");
	//fprintf(eingabe,"set parametric\n");
	fprintf(eingabe,"set terminal x11 persist\n");
	fprintf(eingabe,"set samples 500\n");
	fprintf(eingabe,"set title 'Phase Space Plot von %s vs. %s'\n",y,x);
	fprintf(eingabe,"set xlabel '%si [cm/s]'\n",x);
	fprintf(eingabe,"set ylabel '%si [cm/s]'\n",y);
	fprintf(eingabe,"set xrange [%lf:%lf]\n",-max_x-1.0,max_x+1.0);
	fprintf(eingabe,"set yrange [%lf:%lf]\n",-max_y-1.0,max_y+1.0);
	fprintf(eingabe,"a=%lf\n",a);
	fprintf(eingabe,"b=%lf\n",b);
	fprintf(eingabe,"f(x)=sin(%lf)*x+cos(%lf)*((1-(x*x)/(a*a))*b*b)**0.5\n",phi,phi);
	fprintf(eingabe,"g(x)=sin(%lf)*x+cos(%lf)*-((1-(x*x)/(a*a))*b*b)**0.5\n",phi,phi);
	fprintf(eingabe,"plot f(x) with line 1,g(x) with line 1,'%s' using 1:2 with points\n",datendatei);
	fclose(eingabe);
	sprintf(plotbefehl,"gnuplot %s",datei);
	system(plotbefehl);
}

//Darstellung der Datenzeitreihe
void plotprint2(char datei[], char datendatei[], char x[], char y[], double max_x, double max_y){
	FILE *eingabe;
	char plotbefehl[100];
	eingabe=fopen(datei,"w");
	fprintf(eingabe,"reset\n");
	fprintf(eingabe,"set terminal x11 persist\n");
	fprintf(eingabe,"set title 'Datenreihe von %s vs. %s'\n",y,x);
	fprintf(eingabe,"set xlabel '%si [-]'\n",x);
	fprintf(eingabe,"set ylabel '%si [cm/s]'\n",y);
	fprintf(eingabe,"set xrange [0:%lf]\n",max_x+0.04);
	fprintf(eingabe,"set yrange [%lf:%lf]\n",-max_y-2.0,max_y+2.0);
	fprintf(eingabe,"plot '%s' using 1:2 with line\n",datendatei);
	fclose(eingabe);
	sprintf(plotbefehl,"gnuplot %s",datei);
	system(plotbefehl);
}

//Hauptprogramm in dem erst die Luecken identifiziert und anschliessend interpoliert werden (Ersten und letzten beiden Werte duerfen keine Ausreisser sein!)

main(int argc, char *argv[]){
	
	FILE *eingabe, *ausgabe;
	FILE *plot1, *plot2, *plot3, *plot4;
	
	int punkt, i, j, k, l, n, n_l, n_v, n_dv, n_d2v, l_anfang, l_ende,imuell;
	int weiter,komponente,fehler,schleife,count;
	char lamelle[10], lamellenpkt[3], zeile[200],ut_datei[100], datenpfad[PFAD],bildpfad[PFAD],cmuell[10];
	char plotpath_1[100],plotpath_2[100],plotpath_3[100],plotpath_4[100],udu_datei[100], dud2u_datei[100], ud2u_datei[100];
	char bild1[PFAD],bild2[PFAD],bild3[PFAD],bild4[PFAD],bild5[PFAD],bild6[PFAD],bild7[PFAD],bild8[PFAD];
	double S_v, S_dv, S_d2v, M_v, M_dv, M_d2v, S_vl, S_dvl, S_d2vl, Q_u, Q_du, Q_d2u, Q_v, Q_dv, Q_d2v, S_ud2u, u_teta, d2u_teta;
	double Sigma_u, Sigma_du, Sigma_d2u, lambda_u, lambda_du, lambda_d2u, max_u, max_du, max_d2u, max_v, max_dv, max_d2v;
	double  corx[MAX_DATA],cory[MAX_DATA],corz[MAX_DATA],teta, a, b, Krit1, Krit2, Krit3;
	double v[MAX_DATA],vx[MAX_DATA],vy[MAX_DATA],vz[MAX_DATA];
	double dv[MAX_DATA],d2v[MAX_DATA],u[MAX_DATA],du[MAX_DATA],d2u[MAX_DATA], v_luecke[MAX_DATA];
	float y2[MAX_DATA], x[MAX_DATA], y[MAX_DATA], vi, a_1, a_n;
	
	spike=0;			//Initialwerte fuer Spike-Schleife und Bilderspeicherplatz
	bild=0;
	
	sprintf(pfad,"%s",argv[1]);		//Uebernahmeparameter: Basispfad
	strcpy(bildpfad,pfad);			//Kopieren fuer Bilderspeicherplatz
	strcpy(datenpfad,pfad);				//Kopieren oeffenen und auslesen der Lamelle Punkt + Komponente
	strcat(datenpfad,"/PSTdaten.txt");
	eingabe=fopen(datenpfad,"r");
	fgets(zeile,200,eingabe);
	sscanf(zeile,"%s%i%i",&lamelle,&punkt,&komponente);
	fclose(eingabe);
	strcat(pfad,"/");
	strcat(pfad,lamelle);
	sprintf(lamellenpkt,"/%i_Probe_1_(A820).vel",punkt);
	strcat(pfad,lamellenpkt);
	
	eingabe=fopen(pfad,"r");		//Oeffenen der Punktdatei
	
/*+++++++++++++++++Einlesen der Roh-.vel-Datei+++++++++++++++++++++++++++++++*/

	fgets(zeile,200,eingabe);
	if(sscanf(zeile,"%lf%lf%lf%lf%lf%lf",&vx[0],&vy[0],&vz[0],&corx[0],&cory[0],&corz[0])==6) fseek(eingabe,0L,SEEK_SET);
	else if(sscanf(zeile,"%s%s%i%s%s",&cmuell,&cmuell,&imuell,&cmuell,&cmuell)==5) exit(0);
	i=1; n=0;
	while(!feof(eingabe)){
		fgets(zeile,200,eingabe);
		weiter=feof(eingabe);
		if(weiter==0){
			sscanf(zeile,"%lf%lf%lf%lf%lf%lf",&vx[i],&vy[i],&vz[i],&corx[i],&cory[i],&corz[i]);
			switch(komponente){
				case 1:
					v[i]=vx[i];
					break;
				case 2:
					v[i]=vy[i];
					break;
				case 3:
					v[i]=vz[i];
					break;
				}
			i++;
			n++;
		}
	}
	fclose(eingabe);
	
	S_v=0.0; S_dv=0.0; S_d2v=0.0;
	for(i=1;i<=n;i++){
		S_v+=v[i];
		if(i>1 && i<n){
			dv[i]=(v[i+1]-v[i-1])/2;
			S_dv+=dv[i];
		}
		if(i>2 && i<n-1){
			d2v[i]=(dv[i+1]-dv[i-1])/2;
			S_d2v+=d2v[i];
		}
	}
	M_v=S_v/n;
	M_dv=S_dv/(n-2);
	M_d2v=S_d2v/(n-4);
	
/***Schleife, wenn optische Kontrolle dazu fuehrt, dass weitere Ersetzungen /Luecken folgen sollen***/
	
	schleife=0;										//Initialwerte fuer Schleifenzaehler
	
	while(spike==0){
		
		if(schleife>0)							//Loeschen des vorigen Dialoges
			gtk_widget_destroy(window_PST);
		schleife++;
		
		S_vl=0.0; S_dvl=0.0; S_d2vl=0.0; n_l=0;
		for(i=3;i<=n-2;i++){							//Summe der Ausreisserwerte
			if(v[i]==v_luecke[i]){
				S_vl+=v[i];
				S_dvl+=dv[i];
				S_d2vl+=d2v[i];
				n_l++;
			}
		}
		n_v=n-n_l;
		n_dv=n-n_l-2;
		n_d2v=n-n_l-4;
		M_v=(S_v-S_vl)/n_v;							//Mittelwerte nach Bereinigung
		M_dv=(S_dv-S_dvl)/n_dv;
		M_d2v=(S_d2v-S_d2vl)/n_d2v;
		
		strcpy(udu_datei,bildpfad);					//Schreiben der Plotdaten in 1.Basispfad 2.Vorgabepfad
		strcpy(dud2u_datei,bildpfad);
		strcpy(ud2u_datei,bildpfad);
		strcpy(ut_datei,bildpfad);
		strcat(udu_datei,"/u_du.plt");
		strcat(dud2u_datei,"/du_d2u.plt");
		strcat(ud2u_datei,"/u_d2u.plt");
		strcat(ut_datei,"/ut_datei.plt");
		plot1=fopen(udu_datei,"w");
		plot2=fopen(dud2u_datei,"w");
		plot3=fopen(ud2u_datei,"w");
		plot4=fopen(ut_datei,"w");
		
		Q_u=0.0; Q_du=0.0; Q_d2u=0.0; S_ud2u=0.0; max_v=0.0; max_dv=0.0; max_d2v=0.0;
		for(i=3;i<=n-2;i++){						//Abzug der Mittelwerte und Aufsummierung der Quadrate
			if(v[i]!=v_luecke[i]){
				u[i]=v[i]-M_v;
				Q_v=u[i]*u[i];
				Q_u+=Q_v;
				if(max_v<sqrt(Q_v)) max_v=sqrt(Q_v);
				du[i]=dv[i]-M_dv;
				Q_dv=du[i]*du[i];
				Q_du+=Q_dv;
				if(max_dv<sqrt(Q_dv)) max_dv=sqrt(Q_dv);
				d2u[i]=d2v[i]-M_d2v;
				Q_d2v=d2u[i]*d2u[i];
				Q_d2u+=Q_d2v;
				if(max_d2v<sqrt(Q_d2v)) max_d2v=sqrt(Q_d2v);
				S_ud2u+=u[i]*d2u[i];
				fprintf(plot1,"%lf\t%lf\n",u[i],du[i]);
				fprintf(plot2,"%lf\t%lf\n",du[i],d2u[i]);
				fprintf(plot3,"%lf\t%lf\n",u[i],d2u[i]);
				fprintf(plot4,"%i\t%lf\n",i,u[i]);
			}
		}
		fclose(plot1);
		fclose(plot2);
		fclose(plot3);
		fclose(plot4);
		
		Sigma_u=sqrt(Q_u/(n_v-5));					//Standardabweichung
		Sigma_du=sqrt(Q_du/(n_dv-3));
		Sigma_d2u=sqrt(Q_d2u/(n_d2v-1));
		
		lambda_u=sqrt(2*log(n_v));					//Universelles Kriterium (normalverteilt)
		lambda_du=sqrt(2*log(n_dv));
		lambda_d2u=sqrt(2*log(n_d2v));
		
		max_u=lambda_u*Sigma_u;						//Berechnung der Halbachsen bzw. Maxi- und Minima
		max_du=lambda_du*Sigma_du;
		max_d2u=lambda_d2u*Sigma_d2u;
		
		teta=atan2(S_ud2u,Q_u);		//Berechnung des Drehwinkels der Hauptachsen ueber Schwerpunkt der Punktwolke
		
		b=(max_u*tan(teta)-max_d2u)/(sin(teta)*tan(teta)-cos(teta));	//Berechnung der Halbachsen a und b der u-d2u Ellipse
		a=max_u/cos(teta)-b*tan(teta);
		
/****Steuerungsanweisung fuer GNUPLOT in temporaere Dateien und Darstellung der Ergebnisse in Graphik****/
		strcpy(plotpath_1,bildpfad);
		strcat(plotpath_1,"/PST_1.plt");
		plotprint(plotpath_1,udu_datei,"v","dv",max_v,max_dv,max_u,max_du,0.0);
		strcpy(plotpath_2,bildpfad);
		strcat(plotpath_2,"/PST_2.plt");
		plotprint(plotpath_2,dud2u_datei,"dv","d2v",max_dv,max_d2v,max_du,max_d2u,0.0);
		strcpy(plotpath_3,bildpfad);
		strcat(plotpath_3,"/PST_3.plt");
		plotprint(plotpath_3,ud2u_datei,"v","d2v",max_v,max_d2v,a,b,teta);
		strcpy(plotpath_4,bildpfad);
		strcat(plotpath_4,"/PST_4.plt");
		plotprint2(plotpath_4,ut_datei,"samples","v",n,max_v);
		
		bildspeicherpfad=&bildpfad;			//Uebergabe Bilderspeicherplatz fuer BildEntry
		
/**************** Starten GTK Dialog und Uebernahme von Parametern *********************/
		
		gtk_init(&argc,&argv);
		setlocale(LC_ALL,"C");
		window_PST=create_window_PST();
		gtk_widget_show(window_PST);
		gtk_main();

/************** Loeschen der Umschreiben der Plotdateien *******************************/
		
		if(bild==0){
			remove(udu_datei);
			remove(dud2u_datei);
			remove(ud2u_datei);
			remove(ut_datei);
			remove(plotpath_1);
			remove(plotpath_2);
			remove(plotpath_3);
			remove(plotpath_4);
		}
		if(bild==1){
		
			strcpy(bild1,bildpfad_neu);
			strcat(bild1,"/udu.plt");
			fehler=rename(udu_datei,bild1);
			printf("%i",fehler);
			strcpy(bild2,bildpfad_neu);
			strcat(bild2,"/dud2u.plt");
			rename(dud2u_datei,bild2);
			strcpy(bild3,bildpfad_neu);
			strcat(bild3,"/ud2u.plt");
			rename(ud2u_datei,bild3);
			strcpy(bild4,bildpfad_neu);
			strcat(bild4,"/ut.plt");
			rename(ut_datei,bild4);
			strcpy(bild5,bildpfad_neu);
			strcat(bild5,"/pst1.plt");
			rename(plotpath_1,bild5);
			strcpy(bild6,bildpfad_neu);
			strcat(bild6,"/pst2.plt");
			rename(plotpath_1,bild6);
			strcpy(bild7,bildpfad_neu);
			strcat(bild7,"/pst3.plt");
			rename(ut_datei,bild7);
			strcpy(plotpath_1,bildpfad_neu);
			strcat(bild8,"/pst4.plt");
			rename(plotpath_1,bild8);
		}
			
		if(spike==1) break;		//Aussstieg falls zuletzt dargestellten "Spikes" nicht ausgewiesen werden sollen
		
/********** Ermittlung der Schwellenwerte und Identifizierung von Spikes **************/
		
		count=0;
		for(i=3;i<=n-2;i++){
			u_teta=cos(teta)*u[i]-sin(teta)*d2u[i];
			d2u_teta=sin(teta)*u[i]+cos(teta)*d2u[i];
			Krit1=(u[i]*u[i])/(max_u*max_u)+(du[i]*du[i])/(max_du*max_du);
			Krit2=(du[i]*du[i])/(max_du*max_du)+(d2u[i]*d2u[i])/(max_d2u*max_d2u);
			Krit3=(u_teta*u_teta)/(a*a)+(d2u_teta*d2u_teta)/(b*b);
			
			if(Krit1>1.0||Krit2>1.0||Krit3>1.0||corx[i]<70.0||cory[i]<70.0||corz[i]<70.0){			//Identifizierung als Ausreisser
				v_luecke[i]=v[i];
				count++;
			}
		}
	}
	
/*********** Spline-Interpolation nach PRESS et.al., 1992 *************************/
	
	for(i=1;i<=n;i++){				//Beginn erst, wenn alle Spike-Schleife beendet
		if(v[i]==v_luecke[i]){
			l_anfang=i;
			l_ende=l_anfang;
			while(v[l_ende]==v_luecke[l_ende]) l_ende++;
			j=l_anfang-3; k=0;
			while(j<l_anfang && v[j]!=v_luecke[j]){
				k++;
				x[k]=(float)j;
				y[k]=(float)v[j];
				j++;
			}
			j=l_ende;
			while(j<l_ende+3 && v[j]!=v_luecke[j]){
				k++;
				x[k]=(float)j;
				y[k]=(float)v[j];
				j++;
			}
			a_1=(float)dv[l_anfang-3];
			a_n=(float)dv[l_ende+2];
			spline(x,y,k,a_1,a_n,y2);
			for(l=l_anfang;l<l_ende;l++){
				vi=splint(x,y,y2,k,l);
				v[l]=(double)vi;
			}
		}
	}
	
/************ Schreiben der neuen vel-Datei ********************/

	ausgabe=fopen(speicherpfad,"w");
	if(count/n<0.1) fprintf(ausgabe,"\n");
	else fprintf(ausgabe,"Mehr als 10 %% Fehler\n");
	for(i=1;i<=n;i++){
		switch(komponente){
			case 1:
				vx[i]=v[i];
				break;
			case 2:
				vy[i]=v[i];
				break;
			case 3:
				vz[i]=v[i];
				break;
			}
		fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n",vx[i],vy[i],vz[i],corx[i],cory[i],corz[i]);
	}
	fclose(ausgabe);
	gtk_widget_destroy(window_PST);		//Loeschen des letzten Dialogs
}
