/**********************************************HAYAMI-METROPOLIS-CTR************************************************/

/*      Programm zur Optimierung von Stoffausbreitungsparametern (Dispersionskoeffizient, Tracergeschwindigkeit,
	Wiederfindungsrate der unteren Tracer-Messstation) auf Basis einer Uebertragungsfunktion.
	Analytisches eindimensionales Dispersionsmodell: Hayami-Loesung -> Rutherford (1994)
	Optimierungsalgorithmus: Metropolis-Algorithmus - Simulated Annealing -> Goebels (2007)
		Teile dieses Algorithmus stammen mit Zustimmung von Herrn Dr. Rene Dechow, Max-Planck-Institut Jena,
		wofuer ihm an dieser Stelle herzlich gedankt wird.
	Eingabe: txt-Datei der Messergebnisse zweier Konzentrationsmessungen mit räumlicher Distanz dx
	(Format) Zeit1 Konzentration1 Zeit2 Konzentration2
	+ Informationen zu den Messungen (Abfrage)
	Ausgabe: txt-Dateien der modellierten Konzentrationsverteilung mit optimierten Parametern
	+ txt-Datei der Parameter und Suchwertverlauf
	+ png-Datei des Optimierungsverlaufs
	Interaktion während der Iteration zur Veränderung des Abbruch-Schwellenwertes: Wertanpassung in *_strg.txt
*/

//ACHTUNG: Zeitschrittweite 1 oder 10 sec? => Umstellung in hayamicorr()

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>

#define N 10000
#define PFAD 256
#define PI 3.14159265

double T =5.0;// annealing temperature 
double multipli=0.03;// Multiplikationsfaktor zur Bestimmung der Entfernung neuer (zuf�liger) Parameterwerte 
						// vom Ursprungswert
double rand_max=32767; // 0--RAND_MAX ist der Bereich der Zufallszahlengenerierung (ist eine Konstante!!)
double E1=0;
double par[4],par1[4]; //Vektoren, welche die Parameter enthalten 
double result[N]; // Ergebnisse der Modellrechnungen
double obs[N]; // Observationen= die entsprechenden Messungen
int number; // Nummer der Messwert_ Modellwert Paare
int no_para=3;	//Parameteranzahl: r2,kx,u
char eingabepfad[PFAD],ausgabepfad[PFAD],strgdatei[PFAD];
double dx,r1;	//Entfernung zwischen den Messstationen und Wiederfindungsrate der oberen
double c1[N],diff_t;	//Konzentrationsverteilung oben, Zeitdifferenz
int iter_count2;
double Stat_opt[5];
short m;		//Wert der random-Funktion
double threshold; //Grenzwert der Fehlerquadratsumme
int lfq; 	//Kriterium logarithmierte Fehlerquadrate (zB bei grosser Streuung im hoeheren Konzentrationsbereich)

//Eigentliche Uebertragungsfunktion der Hayami-Lsg
void hayamicorr(int write){
	
	int tau,tau_max,i;
	double dt,p,c_cal,c_teil[N],c_dt;
	double qr, c_grenz;
	
	
	qr=par[0]/r1;	//Verhaeltnis der Wiederfindungsraten
	for(i=0;i<number;i++){		//Ueber alle Zeitschritte der Konzentrationsverteilung...
		if(write==0) c_grenz=obs[i];
		else c_grenz=0.0;
		if(i<number&&c_grenz<1.7){	//...nur wenn ein gueltiger Messwert vorhanden ist (sonst mit 10 zu kennzeichnen)...
		
			c_cal=0.0;
			tau_max=diff_t/10+i;	//Zeitschrittweite 10 sec.
			//tau_max=diff_t+i;	//Zeitschrittweite 1 sec.
			
			for(tau=0;tau<tau_max;tau++){		//...aus allen gemessenen Konzentrationen der oberen Messstation bis zum Zeitpunkt t =tau_max
				dt=diff_t+10.0*(i-tau);	//Zeitschrittweite 10 sec.
				//dt=diff_t+(i-tau);	//Zeitschrittweite 1 sec.
				p=((dx-par[2]*dt)*(dx-par[2]*dt))/(4.0*par[1]*dt);
				c_teil[tau]=qr*c1[tau]*dx/(dt*sqrt(4.0*PI*par[1]*dt))*exp(-p);
				if(tau>0) c_dt=(c_teil[tau-1]+c_teil[tau])/2.0*10.0;	// *10.0 Zeitschrittweite 10 sec., *1.0 Zeitschrittweite 1 sec.
				c_cal+=c_dt;			//...ergibt sich die berechnete Konzentration der unteren Messstation
			}
			result[i]=c_cal;
		}
		else{
			result[i]=obs[i];
		}
	}
}

// Unterprogramm das wieder die alten Parameter aus par1 [x] nach par[x] holt

void back_para (){

	int z2;
	for(z2=0;z2<no_para;z2++){
		par[z2]=par1[z2];
	}
}

//Unterprogramm zur Neukalkulation zufallsbedingter Parameter, �ergabe von y an dieses Programm -->x
void renew_para(double x){
	
	int z2;
	double dummy=0, dummy2=0;
	for(z2=0;z2<no_para;z2++){ //
			// par1 kriegt die derzeitigen Werte von par
		par1[z2]=par[z2];
		m=rand();
		dummy=(2*m/rand_max)-1;// Zufallszahl zwischen -1 und 1
		m=rand();
		dummy2=(x*m/rand_max); // dito
			// zu den ursprnglichen Parametern wird eine Zufallszahl addiert
		par[z2]=par[z2]+dummy*dummy2;
	}
}

// Unterprogramm dass die Optimumsfunktion berechnet, Summe der quadrierten Residuen 

double optimum_func(){

	int y;
	double w=0;

	for(y=0;y<number;y++){
		if(lfq==1) w=w+(log(result[y])-log(obs[y]))*(log(result[y])-log(obs[y]));
		else  w=w+(result[y]-obs[y])*(result[y]-obs[y]);
	}
	if(w<threshold) iter_count2=1;
	
	return w;
}

// Subprogramm metropolis, die ergebenden Werte entsprechen iter_count und iter_count1
int metropolis(int x, int x1){
	
	//Variablendefinition; E1: global
	double E2=0, dE=0, P=0;
	double dummy,y;

	if(x==1){
	// optimum_func ermittelt die Anpassung (Optimumfunktion), beim ersten Durchlauf kriegt E1 den Wert sonst immer E2 (Else)
	// und E1 ist die Anpassung des vorherigen Durchlaufs
		E1=optimum_func();
	}
	else{
		E2=optimum_func ();
		dE=E2-E1;
		if(dE<0){
			P=1; 
			E1=E2;
			x1=0;// neues Ergebnis ist  besser als das alte
		}
		else{
			dummy=rand()/rand_max; // erzeugt eine gleichverteilte Zufallszahl zwischen 0 und 1
			P=exp(-dE/T);		// P ist ein gewichteter Wert fr das Ausmass der Anpassungsgte, je kleiner
						// um so schlimmer
			if(P>dummy){
			// Hier wird der neue Parametersatz trotz Verschlechterung akzeptiert (um lokale Minima zu berspringen)
				E1=E2;x1=0;
			}
			else{
			// hier wird der neue Parametersatz nicht akzeptiert, die neuen Paramter in par[x] werden durch die alten in par2[x]
			// ersetzt
				x1=x1+1;//Parameter verwerfen
				back_para();// back_para ist ein Unterprogramm
			}
		}
	}
	// Nachfolgend wird die "annealing temperature" erniedrigt wenn eine Aenderung des Parametervektors erfolgte
	if(x1==1 || x1==0){
		T=T*0.998;
	}
	// Nachfolgend wird die annealing temperature erhoet und ein Skalierungsfaktor multipli, der die Amplitude der
	// zufaellige Aenderung
	// der Parameter beschreibt erniedrigt, wenn nach 500 Durchlaeufen keine Verbesserung aufgetreten ist
	if(x1>500){
		if(multipli<0.005){
			multipli=0.005;x1=0;
			T=T/0.7;	
		}
		else{
			multipli=multipli/1.5;x1=0;
			T=T/0.7;
		}
	}
	// y ist ein Skalierungsfaktor fuer das Aussmass der zufaelligen Parameteraenderung
	m=rand();
	y=multipli*m/rand_max;
	Stat_opt[1]=E1;Stat_opt[2]=P;Stat_opt[3]=y;
	
	// Nachfolgend Aufruf des Unterprogramms renew_para (y), dass neue zuf�lige Parameter berechnet, �ergabe von y
	renew_para(y);
	// zurck zum Unterprogramm annealing mit x1 --> iter_count1
	return x1;
}




void annealing(){
// Variablendefinition
	FILE *eingabe, *ausgabe;
	int iter_count,iter_count1;
	double  opty_dummy=1000;//entscheidet nach wieviel erfolglosen Durchl�fen der Suchbereich erweitert wird
	int y=0;
	char zeile[PFAD];
	
	iter_count=1;iter_count1=0;iter_count2=0;
	
// Optimierungsschleife, die solange l�ft bis eine Bedingung erfllt ist (Differenz zwischen nacheinanderfolgenden Optimierungen ist 
// 10x kleiner als ein bestimmter Wert)
	while(iter_count2<1){
		// Schrittweise Reduktion der "annealing temperature"
		if(y>200){
			T=T*0.998;
			y=0;
		}
		
		hayamicorr(0);		//hayamicorr berechnet mit par[x] die Konzentrationsverteilung result[x]
		
		// Nachfolgend wird das Unterprogramm metropolis aufgerufen, wobei es die Variablen iter_count und
		// iter_count1 aufnimmt und iter_count1 zurckgibt
		iter_count1=metropolis(iter_count,iter_count1);
		
		if(iter_count1==0){
			ausgabe=fopen(ausgabepfad,"a");
			fprintf(ausgabe,"%lf\t%lf\t%lf\n",par[1],par[2],E1);
			fclose(ausgabe);
		}
		// An dieser Stelle werden die aktuellen Anpassungen, etc. auf 
		// der Konsole angezeigt
		printf("Nr:%i Nr:%i T:%lf E:%lf P:%lf y:%lf r2:%lf kx:%lf u:%lf\n",iter_count,iter_count1,T,Stat_opt[1],Stat_opt[2],Stat_opt[3],par[0],par[1],par[2]);
		
		//Einlesen veraenderter Steuerungsparameter, derzeit nur threshold.
		eingabe=fopen(strgdatei,"r");
		fgets(zeile,100,eingabe);
		sscanf(zeile,"%lf",&threshold);
		fclose(eingabe);

		// bei "opty_dummy" erfolglosen Durchl�fen wird Suchbereich erweitert(x multipli)
		if(iter_count1>opty_dummy){
			iter_count1=0;iter_count2=0;multipli=2;
		}
		// iter_count ist die Nummer des aktuellen Durchlaufs und wird um 1 erh�t
		iter_count=iter_count+1;
		y=y+1;
	}
}

//Hauptprogramm: Eingabe der Dateien und Startwerte, Einlesen der C(t), Schreiben und Ausgabe der Ergebnisdateien, Bilder und Werte
int main(){

	FILE *ausgabe, *eingabe;
	char plotdatei[PFAD],pngdatei[PFAD],plotbefehl[PFAD],zeile[PFAD];
	int i, weiter=1;
	double t_dummy[N],tau_dummy[N];
	
	//Abfrage der Speicherpfade und Dateinamen, Distanz zwischen Messstationen, Star- und Messwerte sowie Kontrolldaten der Optimierung

	printf("Dateipfad der Konzentrationsverteilungen:\n");
	gets(eingabepfad);
	printf("Dateipfad der Ergebnisdatei (ohne Endung):\n");
	gets(ausgabepfad);
	strcpy(plotdatei,ausgabepfad);
	strcpy(strgdatei,ausgabepfad);
	strcat(ausgabepfad,".txt");
	printf("Distanz zwischen den Messstationen:\n");
	scanf("%lf",&dx);
	printf("Wiederfindungsrate der Messstation oben:\n");
	scanf("%lf",&r1);
	printf("Startwert Wiederfindungsrate unten:\n");
	scanf("%lf",&par[0]);
	printf("Startwert Dispersionskoeffizienten:\n");
	scanf("%lf",&par[1]);
	printf("Startwert Tracergeschwindigkeit:\n");
	scanf("%lf",&par[2]);
	printf("Logarithmierte Fehlerquadrate? (ja:1/nein:0):\n");
	scanf("%i",&lfq);
	printf("Zielwert der summierten Fehlerquadrate:\n");
	scanf("%lf",&threshold);
	
	strcat(strgdatei,"_strg.txt");		//Anlegen der Steuerungsdatei: Momentan nur fuer Threshold.
	eingabe=fopen(strgdatei,"w");		//zur Veraenderung der Werte waehrend dem Programmlauf.
 	fprintf(eingabe,"%lf\n",threshold);
	fclose(eingabe);

	eingabe=fopen(eingabepfad,"r");
	
	//Einlesen der Messwertdatei
	i=0;number=0;
	while(!feof(eingabe)){
		fgets(zeile,200,eingabe);
		
		if(!feof(eingabe)){
			sscanf(zeile,"%lf%lf%lf%lf",&tau_dummy[i],&c1[i],&t_dummy[i],&obs[i]);	
			i++;
			number++;
		} 
	}
	fclose(eingabe);

	//Beginn der Optimierungsschleife
	diff_t=t_dummy[0]-tau_dummy[0];
	while(threshold!=0.0){
	
		ausgabe=fopen(ausgabepfad,"w");
		fclose(ausgabe);

		// startet den Optimierungsalgorithmus
		annealing (); 
	
		//Erstellen der Plotdateien: Anweisung:*.plt, Bilddatei: *.png
		strcpy(pngdatei,plotdatei);		
		strcat(pngdatei,".png");
		strcat(plotdatei,".plt");
		eingabe=fopen(plotdatei,"w");		//Plotanweisung
		fprintf(eingabe,"reset\n");
		fprintf(eingabe,"set terminal x11 persist\n");
		fprintf(eingabe,"set title 'kx und U ber Hayami-Routing'\n");
		fprintf(eingabe,"set xlabel 'kx [m2/s]'\n");
		fprintf(eingabe,"set ylabel 'U [m/s]'\n");
		fprintf(eingabe,"set zlabel 'Sum(C2gem.-C2hayami)^2 [g^2/m^6]'\n");
		fprintf(eingabe,"set view 50,10,1.0,1.0\n");
		fprintf(eingabe,"splot '%s'with line\n",ausgabepfad);
		fprintf(eingabe,"set output '%s'\n",pngdatei);		//Erstellung Bilddatei
		fprintf(eingabe,"set terminal png\n");
		fprintf(eingabe,"replot\n");
		fclose(eingabe);
		sprintf(plotbefehl,"gnuplot %s",plotdatei);
		system(plotbefehl);
		remove(plotdatei);
	
		//Interaktion mit Steuerungsdatei, wenn Zielwert erreicht
		printf("\nErgebnis: kx: %lf\tu: %lf\tr2: %lf\n",par[1],par[2],par[0]);
		printf("Neuer Zielwert der summierten Fehlerquadrate (aktuell:%lf)?\n",threshold);
		printf("Oder Optimierung mit 0 beenden:\n");
		scanf("%lf",&threshold);		
	}

	//Schreiben der gemessenen und gerouteten Konzentrationsverteilungen
	ausgabe=fopen(eingabepfad,"w");		
	for(i=0;i<number;i++){
		if(obs[i]==result[i]) hayamicorr(1);
		fprintf(ausgabe,"%.0lf\t%.16lf\t%.0lf\t%.16lf\t%.16lf\n",tau_dummy[i],c1[i],t_dummy[i],obs[i],result[i]);
	}
	fclose(ausgabe);
	
	ausgabe=fopen(ausgabepfad,"a");
	fprintf(ausgabe,"kx:%lf\tu:%lf\tr2:%lf\n",par[1],par[2],par[0]);
	fclose(ausgabe);
}
